#include "vimeoaccountslistmodel.h"
#include "vimeo.h"
#include "notifications.h"
#include <QSqlRecord>
#include <QSqlField>

VimeoAccountsListModel::VimeoAccountsListModel(QObject *parent) :
    QSqlTableModel(parent, QSqlDatabase::database())
{
    m_roleNames[DisplayNameRole] = "displayName";
    m_roleNames[UsernameRole] = "username";
    m_roleNames[AccessTokenRole] = "accessToken";
    m_roleNames[TokenSecretRole] = "tokenSecret";
    m_roleNames[ActiveRole] = "active";
#if QT_VERSION < 0x050000
    this->setRoleNames(m_roleNames);
#endif
    this->setTable("vimeoAccounts");
    this->setEditStrategy(QSqlTableModel::OnManualSubmit);
    this->select();

    this->connect(Vimeo::instance(), SIGNAL(signedIn(QString,QString,QString)), this, SLOT(addAccount(QString,QString,QString)));
    this->connect(this, SIGNAL(accountSelected(QString,QString,QString)), Vimeo::instance(), SLOT(setAccount(QString,QString,QString)));
    this->connect(this, SIGNAL(accountAdded(QString,QString,QString)), Vimeo::instance(), SLOT(setAccount(QString,QString,QString)));
    this->connect(this, SIGNAL(accountDeleted()), Vimeo::instance(), SLOT(setAccount()));
    this->connect(this, SIGNAL(info(QString)), Notifications::instance(), SLOT(onInfo(QString)));
    this->connect(this, SIGNAL(error(QString)), Notifications::instance(), SLOT(onError(QString)));
}

VimeoAccountsListModel::~VimeoAccountsListModel() {
    this->clear();
}

#if QT_VERSION >= 0x050000
QHash<int, QByteArray> VimeoAccountsListModel::roleNames() const {
    return m_roleNames;
}
#endif

void VimeoAccountsListModel::setActiveAccount(int row) {
    this->database().open();
    QSqlField nameField("name", QVariant::String);
    QSqlField usernameField("username", QVariant::String);
    QSqlField tokenField("token", QVariant::String);
    QSqlField secretField("secret", QVariant::String);
    QSqlField activeField("active", QVariant::Int);
    nameField.setValue(this->data(this->index(row, 0), DisplayNameRole));
    usernameField.setValue(this->data(this->index(row, 0), UsernameRole));
    tokenField.setValue(this->data(this->index(row, 0), AccessTokenRole));
    secretField.setValue(this->data(this->index(row, 0), TokenSecretRole));
    activeField.setValue(1);

    QSqlRecord rec;
    rec.append(nameField);
    rec.append(usernameField);
    rec.append(tokenField);
    rec.append(secretField);
    rec.append(activeField);

    this->setRecord(row, rec);

    for (int i = 0; i < this->rowCount(); i++) {
        if (i != row) {
            nameField.setValue(this->data(this->index(i, 0), DisplayNameRole));
            usernameField.setValue(this->data(this->index(i, 0), UsernameRole));
            tokenField.setValue(this->data(this->index(i, 0), AccessTokenRole));
            secretField.setValue(this->data(this->index(i, 0), TokenSecretRole));
            activeField.setValue(0);

            rec.clear();
            rec.append(nameField);
            rec.append(usernameField);
            rec.append(tokenField);
            rec.append(secretField);
            rec.append(activeField);

            this->setRecord(i, rec);
        }
    }

    this->submitAll();
}

void VimeoAccountsListModel::switchAccount(int row) {
    this->setActiveAccount(row);

    this->database().open();
    QString user = this->data(this->index(row, 0), UsernameRole).toString();
    QString token = this->data(this->index(row, 0), AccessTokenRole).toString();
    QString secret = this->data(this->index(row, 0), TokenSecretRole).toString();

    emit accountSelected(user, token, secret);
}

QVariant VimeoAccountsListModel::data(const QModelIndex &idx, int role) const {
    return QSqlTableModel::data(this->index(idx.row(), role));
}

QVariant VimeoAccountsListModel::data(int row, const QByteArray &role) const {
    return this->data(this->index(row, 0), this->roleNames().key(role));
}

void VimeoAccountsListModel::addAccount(const QString &displayName, const QString &token, const QString &secret) {
    this->database().open();
    QSqlField nameField("name", QVariant::String);
    QSqlField usernameField("username", QVariant::String);
    QSqlField tokenField("token", QVariant::String);
    QSqlField secretField("secret", QVariant::String);
    QSqlField activeField("active", QVariant::Int);
    nameField.setValue(displayName);
    usernameField.setValue(QVariant::String);
    tokenField.setValue(token);
    secretField.setValue(secret);
    activeField.setValue(0);

    QSqlRecord rec;
    rec.append(nameField);
    rec.append(usernameField);
    rec.append(tokenField);
    rec.append(secretField);
    rec.append(activeField);

    this->insertRecord(-1, rec);
    this->setActiveAccount(this->rowCount() - 1);

    emit accountAdded(QString(), token, secret);
    emit countChanged(this->rowCount());
}

void VimeoAccountsListModel::deleteAccount(int row) {
    this->database().open();
    bool signOut = this->data(this->index(row, 0), ActiveRole).toBool();

    if ((this->removeRow(row)) && (this->submitAll())) {
        if (signOut) {
            emit accountDeleted();
        }

        emit countChanged(this->rowCount());
        emit info(tr("Account deleted. Please visit the Vimeo website to revoke access"));
    }
    else {
        emit error(tr("Database error. Unable to delete account"));
    }
}
